import settings as sfs_settings
import models
from django.core.urlresolvers import reverse
from django.shortcuts import render
from django.utils import ipv6

class StopForumSpamMiddleware():

    def process_request(self, request):
        
        if sfs_settings.FORCE_ALL_REQUESTS:
            return self.check_request_ip(request)
        
        def compile_paths(path_list):
            paths = []
            for path in path_list:
                if path.startswith("/"):
                    paths.append(path)
                else:
                    paths.append(reverse(path))
            return paths
        
        if not request.method == 'POST':
            return
        
        if sfs_settings.ALL_POST_REQUESTS:
            if request.path in compile_paths(sfs_settings.URLS_IGNORE):
                return
            return self.check_request_ip(request)
        
        if request.path in compile_paths(sfs_settings.URLS_INCLUDE):
            return self.check_request_ip(request)
    
    def check_request_ip(self, request):
        
        remote_ip = ipv6._unpack_ipv4(request.META['REMOTE_ADDR'])
        
        cache_entries = models.Cache.objects.filter(ip=remote_ip)
        
        if cache_entries.count() > 0:
            if sfs_settings.LOG_SPAM:
                log = models.Log(message = "Spam received from %s" % remote_ip)
                log.save()
            
            return render(request, 'stopforumspam/denied.html', {"cache_entries": cache_entries,},
                          status=403)
    



from django.db import transaction
from django.core.management.base import BaseCommand

try:
    from django.utils import timezone
except ImportError:
    from datetime import datetime as timezone

from stopforumspam import models
from stopforumspam import settings as sfs_settings
from django.conf import settings

import re
import urllib
import zipfile
from optparse import make_option

def safe_bulk_create(objs):
    """Wrapper to overcome the size limitation of standard bulk_create()"""
    if len(objs) == 0: return
    if not 'sqlite' in settings.DATABASES['default']['ENGINE']:
        objs[0].__class__.objects.bulk_create(objs)
    else:
        BULK_SIZE = 900/len(objs[0].__class__._meta.fields)
        for i in range(0,len(objs),BULK_SIZE):
            print "SQLITE size limits: Bulk inserting %d objects" % BULK_SIZE
            objs[0].__class__.objects.bulk_create(objs[i:i+BULK_SIZE])

class Command(BaseCommand):
    args = '--force'
    help = 'Updates the database with the latest IPs from stopforumspam.com'
    option_list = BaseCommand.option_list + (
        make_option('--force', '-f', dest='force', default=False,
                    action='store_true',
                    help='Force update of options'),
        )
    
    def handle(self, *args, **options):
        self.ensure_updated(options['force'])
        
    def ensure_updated(self, force=False):
        last_update = models.Log.objects.filter(message=sfs_settings.LOG_MESSAGE_UPDATE).order_by('-inserted')
        do_update = force
        if not do_update and last_update.count() > 0:
            days_ago = timezone.now() - last_update[0].inserted
            if days_ago.days >= sfs_settings.CACHE_EXPIRE:
                do_update = True
        else:
            do_update = True
        if do_update:
            print "Updating (this may take some time)"
            print "If you abort this command and want to rebuild, you have to use the --force option!"
            self.do_update()
        else:
            print "Nothing to update"
    
    
    @transaction.commit_manually()
    def do_update(self):
        try:
            self._do_update()
            transaction.commit()
        except Exception:
            transaction.rollback()
            raise

    def _do_update(self):
        # Delete old cache
        models.Cache.objects.filter(permanent=False).delete()
        
        # First log the update
        log = models.Log()
        log.message = sfs_settings.LOG_MESSAGE_UPDATE
        log.save()
        
        # For security purposes we test that each line is actually an IP address
        ip_match = re.compile(r"^(\d+)\.(\d+)\.(\d+)\.(\d+)$")
        
        if sfs_settings.SOURCE_ZIP.startswith("file://"):
            filename = sfs_settings.SOURCE_ZIP.split("file://")[1]
        else:
            filename, __ = urllib.urlretrieve(sfs_settings.SOURCE_ZIP)
        z = zipfile.ZipFile(filename)
        ips = z.read(sfs_settings.ZIP_FILENAME)
        ips = ips.split("\n")
        ips = filter(lambda x: ip_match.match(x), ips)
        inserted = 0
        total = len(ips)
        objects_to_save = []
        for ip in ips:
            cache = models.Cache(ip=ip)
            objects_to_save.append(cache)
            inserted = inserted + 1
            if inserted % 100 == 0:
                print "Object %d of %d found" % (inserted, total)
        
        print "===================="
        print " Saving to database "
        print "===================="

        if hasattr(models.Cache.objects, 'bulk_create'):
            print "New django with bulk_create detected. Inserting everyting at once!"
            safe_bulk_create(objects_to_save)
        else:
            print "Django<1.5, inserting one object at a time..."
            for cnt,cache in enumerate(objects_to_save):
                cache.save()
                if cnt % 100 == 0:
                    print "Object %d of %d saved" % (inserted, total)

    



from setuptools import setup, find_packages
import os

CLASSIFIERS = [
    'Development Status :: 4 - Beta',
    'Environment :: Web Environment',
    'Framework :: Django',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    'Topic :: Software Development',
    'Topic :: Software Development :: Libraries :: Application Frameworks',
]

packages = find_packages()

def build_media_pattern(base_folder, file_extension):
    return ["%s/%s*.%s" % (base_folder, "*/"*x, file_extension) if base_folder else "%s*.%s" % ("*/"*x, file_extension) for x in range(10)]

media_patterns = ( build_media_pattern("templates", "html") +
     	           build_media_pattern("static", "js") +
                   build_media_pattern("static", "css") +
                   build_media_pattern("static", "png") +
                   build_media_pattern("static", "jpeg") +
                   build_media_pattern("static", "gif") +
                   build_media_pattern("", "rst")
)

package_data = dict(
    (package_name, media_patterns)
    for package_name in packages
)

setup(name='stopforumspam',
      description='Django middleware for blocking IPs listed in stopforumspam.com',
      long_description=open(os.path.join(os.path.dirname(__file__), 'README.md')).read(),
      author="Benjamin Bach",
      author_email="benjamin@overtag.dk",
      version='1.4.1',
      packages=packages,
      license='BSD License',
      install_requires=[
        'Django>=1.3.0',
      ],
      url='https://overtag.dk/',
      classifiers=CLASSIFIERS,
      include_package_data=True,
      package_data=package_data,
)



from django.db import models
from datetime import timedelta
#from django.utils import timezone

try:
    from django.utils import timezone
except ImportError:
    from datetime import datetime as timezone

import settings as sfs_settings
from django.utils.translation import ugettext_lazy as _

# Create your models here.
class Cache(models.Model):
    
    updated = models.DateTimeField(auto_now=True, verbose_name=_(u"Updated"))
    ip = models.IPAddressField(verbose_name=_(u"IP address"),)
    permanent = models.BooleanField(default=False, verbose_name=_(u"Permanent"))
    
    class Meta:
        verbose_name = _(u"Cache entry")
        verbose_name_plural = _(u"Cache")
    
class Log(models.Model):
    inserted = models.DateTimeField(auto_now_add=True, verbose_name=_(u"inserted"))    
    message = models.CharField(max_length=1024, verbose_name=_(u"message"))
    def save(self):
        super(Log, self).save()
        # Delete old log messages
        Log.objects.filter(inserted__lt=timezone.now()-timedelta(days=sfs_settings.LOG_EXPIRE)).delete()
    
    class Meta:
        ordering = ('-inserted',)
        verbose_name = _(u"Log message")
        verbose_name_plural = _(u"Log")
